@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -26,6 +25,7 @@
 #include <NetInfo/dsutil.h>
 #include <NetInfo/dsx500.h>
 #include <NetInfo/dsindex.h>
+#include <NetInfo/utf-8.h>
 
 /*
  * N.B.  This is also defined in netinfod/ni_file.c
@@ -49,6 +49,8 @@ dsengine_new(dsengine **s, char *name, u_int32_t flags)
 
 	*s = (dsengine *)malloc(sizeof(dsengine));
 	(*s)->store = x;
+	(*s)->delegate = NULL;
+	(*s)->private = NULL;
 
 	/* create root if necessary */
 	r = dsstore_fetch(x, 0);
@@ -132,6 +134,7 @@ dsengine_detach(dsengine *s, dsrecord *r, u_int32_t dsid)
 	r->index = NULL;
 
 	status = dsstore_save(s->store, r);
+	if (status == DSStatusOK) dsstore_notify(s->store);
 
 	return status;
 }
@@ -158,6 +161,7 @@ dsengine_attach(dsengine *s, dsrecord *r, u_int32_t dsid)
 	r->index = NULL;
 
 	status = dsstore_save(s->store, r);
+	if (status == DSStatusOK) dsstore_notify(s->store);
 
 	return status;
 }
@@ -185,6 +189,7 @@ dsengine_set_parent(dsengine *s, dsrecord *r, u_int32_t dsid)
 
 	r->super = dsid;
 	status = dsstore_save(s->store, r);
+	if (status == DSStatusOK) dsstore_notify(s->store);
 
 	return status;
 }
@@ -222,6 +227,7 @@ dsengine_create(dsengine *s, dsrecord *r, u_int32_t dsid)
 		status = dsstore_save(s->store, parent);
 
 	dsrecord_release(parent);
+	if (status == DSStatusOK) dsstore_notify(s->store);
 
 	return status;
 }
@@ -300,6 +306,7 @@ dsengine_save(dsengine *s, dsrecord *r)
 {
 	u_int32_t i;
 	dsrecord *parent;
+	dsstatus status;
 
 	if (s == NULL) return DSStatusInvalidStore;
 
@@ -318,7 +325,43 @@ dsengine_save(dsengine *s, dsrecord *r)
 	dsindex_insert_record(parent->index, r);
 
 	dsrecord_release(parent);
-	return dsstore_save(s->store, r);
+	status = dsstore_save(s->store, r);
+	if (status == DSStatusOK) dsstore_notify(s->store);
+
+	return status;
+}
+
+/*
+ * Save a record quickly.
+ */
+dsstatus
+dsengine_save_fast(dsengine *s, dsrecord *r)
+{
+	u_int32_t i;
+	dsrecord *parent;
+	dsstatus status;
+
+	if (s == NULL) return DSStatusInvalidStore;
+
+	i = dsstore_record_super(s->store, r->dsid);
+	if (i == IndexNull) return DSStatusInvalidPath;
+
+	parent = dsstore_fetch(s->store, i);
+	if (parent == NULL)
+	{
+		/* XXX Parent doesn't exist! */
+		return DSStatusInvalidPath;
+	}
+
+	/* Update child in parent's index */
+	dsindex_delete_dsid(parent->index, r->dsid);
+	dsindex_insert_record(parent->index, r);
+
+	dsrecord_release(parent);
+	status = dsstore_save_fast(s->store, r, 0);
+	if (status == DSStatusOK) dsstore_notify(s->store);
+
+	return status;
 }
 
 /*
@@ -329,6 +372,7 @@ dsengine_save_attribute(dsengine *s, dsrecord *r, dsattribute *a, u_int32_t asel
 {
 	u_int32_t i;
 	dsrecord *parent;
+	dsstatus status;
 
 	if (s == NULL) return DSStatusInvalidStore;
 
@@ -347,7 +391,10 @@ dsengine_save_attribute(dsengine *s, dsrecord *r, dsattribute *a, u_int32_t asel
 	dsindex_insert_record(parent->index, r);
 
 	dsrecord_release(parent);
-	return dsstore_save_attribute(s->store, r, a, asel);
+	status = dsstore_save_attribute(s->store, r, a, asel);
+	if (status == DSStatusOK) dsstore_notify(s->store);
+
+	return status;
 }
 
 /*
@@ -359,12 +406,17 @@ dsengine_remove(dsengine *s, u_int32_t dsid)
 	u_int32_t i, n, *kids;
 	dsrecord *r, *parent;
 	dsstatus status;
+	char *ns;
 
 	if (s == NULL) return DSStatusInvalidStore;
 
 	r = dsstore_fetch(s->store, dsid);
 	if (r == NULL) return DSStatusOK;
 	
+	/* PROTECT FROM MULTIPLE NOTIFICATIONS */
+	ns = s->store->notification_name;
+	s->store->notification_name = NULL;
+
 	if (r->sub_count > 0)
 	{
 		n = r->sub_count;
@@ -378,6 +430,7 @@ dsengine_remove(dsengine *s, u_int32_t dsid)
 			if (status != DSStatusOK)
 			{
 				free(kids);
+				s->store->notification_name = ns;
 				return status;
 			}
 		}
@@ -386,12 +439,17 @@ dsengine_remove(dsengine *s, u_int32_t dsid)
 	}
 
 	i = dsstore_record_super(s->store, dsid);
-	if (i == IndexNull) return DSStatusInvalidPath;
+	if (i == IndexNull)
+	{
+		s->store->notification_name = ns;
+		return DSStatusInvalidPath;
+	}
 
 	parent = dsstore_fetch(s->store, i);
 	if (parent == NULL)
 	{
 		/* XXX Parent doesn't exist! */
+		s->store->notification_name = ns;
 		return DSStatusInvalidPath;
 	}
 
@@ -410,10 +468,15 @@ dsengine_remove(dsengine *s, u_int32_t dsid)
 	if (status != DSStatusOK)
 	{
 		/* XXX can't save changes to parent! */
+		s->store->notification_name = ns;
 		return status;
 	}
 
 	status = dsstore_remove(s->store, dsid);
+
+	s->store->notification_name = ns;
+	if (status == DSStatusOK) dsstore_notify(s->store);
+
 	return status;
 }
 
@@ -486,6 +549,8 @@ dsengine_move(dsengine *s, u_int32_t dsid, u_int32_t pdsid)
 		return status;
 	}
 
+	dsstore_notify(s->store);
+
 	return DSStatusOK;
 }
 	
@@ -498,6 +563,7 @@ dsengine_copy(dsengine *s, u_int32_t dsid, u_int32_t pdsid)
 	dsrecord *r, *x;
 	u_int32_t *kids, i, count;
 	dsstatus status;
+	char *ns;
 
 	if (s == NULL) return DSStatusInvalidStore;
 	
@@ -512,11 +578,16 @@ dsengine_copy(dsengine *s, u_int32_t dsid, u_int32_t pdsid)
 	x->sub_count = 0;
 	x->sub = NULL;
 
+	/* PROTECT FROM MULTIPLE NOTIFICATIONS */
+	ns = s->store->notification_name;
+	s->store->notification_name = NULL;
+
 	status = dsengine_create(s, x, pdsid);
 	if (status != DSStatusOK)
 	{
 		/* XXX Can't add new record! */
 		dsrecord_release(x);
+		s->store->notification_name = ns;
 		return status;
 	}
 		
@@ -527,10 +598,14 @@ dsengine_copy(dsengine *s, u_int32_t dsid, u_int32_t pdsid)
 		{
 			/* XXX Can't add child! */
 			dsrecord_release(x);
+			s->store->notification_name = ns;
 			return status;
 		}
 	}
 
+	s->store->notification_name = ns;
+	dsstore_notify(s->store);
+
 	return DSStatusOK;
 }
 
@@ -632,6 +707,7 @@ _filter_searcher(dsengine *s, u_int32_t dsid, dsfilter *f, u_int32_t scopemin, u
 	dsrecord *r;
 	u_int32_t i, x;
 	dsstatus status;
+	Logic3 eval;
 
 	if (s == NULL) return DSStatusInvalidStore;
 
@@ -642,7 +718,12 @@ _filter_searcher(dsengine *s, u_int32_t dsid, dsfilter *f, u_int32_t scopemin, u
 
 	if (scopemin == 0)
 	{
-		if (dsfilter_test(f, r) == L3True)
+		if (s->delegate != NULL)
+			eval = (s->delegate)(f, r, s->private);
+		else
+			eval = dsfilter_test(f, r);
+
+		if (eval == L3True)
 		{
 			if (findall == 0)
 			{
@@ -710,15 +791,18 @@ dsengine_match(dsengine *s, u_int32_t dsid, dsdata *key, dsdata *val, u_int32_t
 }
 
 static dsstatus
-dsengine_pathutil(dsengine *s, u_int32_t dsid, dsrecord *path, u_int32_t *match, u_int32_t create)
+dsengine_pathutil(dsengine *s, u_int32_t dsid, dsrecord *path, u_int32_t *match, u_int32_t *create)
 {
-	u_int32_t i, n, c;
+	u_int32_t i, n, c, do_create;
 	dsstatus status;
 	dsattribute *a;
 	dsdata *k, *v;
 	dsrecord *r;
 	dsdata *keyname, *dot, *dotdot;
 
+	do_create = *create;
+	*create = 0;
+
 	*match = (u_int32_t)-1;
 
 	if (s == NULL) return DSStatusInvalidStore;
@@ -775,7 +859,7 @@ dsengine_pathutil(dsengine *s, u_int32_t dsid, dsrecord *path, u_int32_t *match,
 
 		if (c == (u_int32_t)-1)
 		{
-			if (create == 0)
+			if (do_create == 0)
 			{
 				dsdata_release(keyname);
 				dsdata_release(dot);
@@ -785,6 +869,7 @@ dsengine_pathutil(dsengine *s, u_int32_t dsid, dsrecord *path, u_int32_t *match,
 			else
 			{
 				/* Create the path component */
+				*create = 1;
 				r = dsrecord_new();
 				dsrecord_append_attribute(r, a, SELECT_ATTRIBUTE);
 				status = dsengine_create(s, r, n);
@@ -846,7 +931,10 @@ dsengine_path(dsengine *s, u_int32_t dsid, u_int32_t **list)
 dsstatus
 dsengine_pathmatch(dsengine *s, u_int32_t dsid, dsrecord *path, u_int32_t *match)
 {
-	return dsengine_pathutil(s, dsid, path, match, 0);
+	int x;
+
+	x = 0;
+	return dsengine_pathutil(s, dsid, path, match, &x);
 }
 
 /*
@@ -857,7 +945,26 @@ dsengine_pathmatch(dsengine *s, u_int32_t dsid, dsrecord *path, u_int32_t *match
 dsstatus
 dsengine_pathcreate(dsengine *s, u_int32_t dsid, dsrecord *path, u_int32_t *match)
 {
-	return dsengine_pathutil(s, dsid, path, match, 1);
+	dsstatus status;
+	char *ns;
+	u_int32_t create;
+
+	/* PROTECT FROM MULTIPLE NOTIFICATIONS */
+	ns = s->store->notification_name;
+	s->store->notification_name = NULL;
+
+	create = 1;
+	status = dsengine_pathutil(s, dsid, path, match, &create);
+
+	s->store->notification_name = ns;
+
+	if ((status == DSStatusOK) && (create != 0))
+	{
+		/* Send notification */
+		dsstore_notify(s->store);
+	}
+
+	return status;
 }
 
 /*
@@ -876,6 +983,15 @@ dsengine_list(dsengine *s, u_int32_t dsid, dsdata *key, u_int32_t scopemin, u_in
 	dsstatus status;
 	
 	if (s == NULL) return DSStatusInvalidStore;
+	if (list == NULL) return DSStatusFailed;
+
+	*list = NULL;
+
+	if ((scopemin == 1) && (scopemax == 1))
+	{
+		return dsstore_list(s->store, dsid, key, SELECT_ATTRIBUTE, list);
+	}
+
 	if (key == NULL) return DSStatusInvalidKey;
 
 	r = dsrecord_new();
@@ -924,6 +1040,7 @@ dsengine_list(dsengine *s, u_int32_t dsid, dsdata *key, u_int32_t scopemin, u_in
 			a->value[j] = dsdata_retain(r->attribute[x]->value[j]);
 
 		dsrecord_append_attribute(*list, a, SELECT_ATTRIBUTE);
+		dsattribute_release(a);
 		dsrecord_release(r);
 	}
 
@@ -1040,16 +1157,13 @@ dsengine_netinfo_string_path(dsengine *s, u_int32_t dsid)
 			else if (name->count == 0) dirno = 1;
 		}
 
-		if (dirno == 1)
+		if ((dirno == 1) ||
+			((x = dsdata_to_cstring(name->value[0])) == NULL))
 		{
 			if (r == NULL) sprintf(str, "dir:?");
 			else sprintf(str, "dir:%u", r->dsid);
 			x = str;
 		}
-		else
-		{
-			x = dsdata_to_cstring(name->value[0]);
-		}
 
 		len = strlen(x);
 		p = malloc(1 + len + plen + 1);
@@ -1080,35 +1194,36 @@ dsengine_netinfo_string_path(dsengine *s, u_int32_t dsid)
  *
  * Here is the algorithm we use to construct a DN:
  *
- *	for each directory in the name
- *   {
- *		if directory has "_rdn" attribute
- *       {
- *			use attribute named by value of rdn attribute as RDN.
- *       }
- *		else if directory has "name "attribute 
- *       {
- *			use "name" as RDN
- *		}
- *       else
- *       {
- *			use DSID=%s as RDN
- *		}
- *   }
+ * for each directory in the name {
+ *  if directory has "_rdn" attribute {
+ *    use attribute named by value of rdn attribute as RDN.
+ *  } else if directory has "name "attribute {
+ *    use "name" as RDN
+ *  } else {
+ *    use DSID=%s as RDN
+ *  }
+ * }
  *
  * A remaining issue is dealing with the fact that NetInfo
  * names are case sensitive but X.500 names are not. We may
  * need to resort to DSID=%s for all RDNs.
+ *
+ * Also, multi-valued RDNs are not supported. We could support
+ * them by having multiple values for the _rdn attribute (for
+ * example, to form a DN cn=foo+sn=bar,c=US) but there are 
+ * tricky canonicalization issues to be dealt with.
+ *
  */
 char *
 dsengine_x500_string_path(dsengine *s, u_int32_t dsid)
 {
-	char *p, *path, str[64], *x;
+	char *p, *path, str[64], *escaped;
 	u_int32_t plen;
 	dsrecord *r;
 	dsattribute *rdn_attr, *name;
 	dsstatus status;
 	dsdata *rdn_key, *default_rdn_key, *name_rdn, *name_dsid;
+	dsdata *x, tmp;
 
 	if (s == NULL) return NULL;
 	if (dsid == 0) return copyString("");
@@ -1118,7 +1233,7 @@ dsengine_x500_string_path(dsengine *s, u_int32_t dsid)
 
 	default_rdn_key = cstring_to_dsdata("name");
 	name_rdn = cstring_to_dsdata("rdn");
-	name_dsid = cstring_to_dsdata("DSID");
+	name_dsid = casecstring_to_dsdata("DSID");
 
 	status = dsengine_fetch(s, dsid, &r);
 	if (status != DSStatusOK) return NULL;
@@ -1146,38 +1261,48 @@ dsengine_x500_string_path(dsengine *s, u_int32_t dsid)
 			rdn_key = dsdata_retain(name_dsid);
 		}
 
-		x = dsdata_to_cstring(rdn_key);
+		escaped = escape_rdn(rdn_key);
+
 		if (path == NULL)
 		{
-			plen = strlen(x) + 1;
+			plen = strlen(escaped) + 1;
 			path = malloc(plen + 1);
-			sprintf(path, "%s=", x);
+			sprintf(path, "%s=", escaped);
 		}
 		else
 		{
-			plen += strlen(x) + 2;
+			plen += strlen(escaped) + 2;
 			p = malloc(plen + 1);
-			sprintf(p, "%s,%s=", path, x);
+			sprintf(p, "%s,%s=", path, escaped);
 			free(path);
 			path = p;
 		}
 
+		free(escaped);
+
 		if (name == NULL || name->count == 0)
 		{
 			sprintf(str, "%u", r->dsid);
-			x = str;
+			tmp.type = DataTypeCStr;
+			tmp.length = strlen(str) + 1;
+			tmp.data = str;
+			tmp.retain = 1;
+			x = &tmp;
 		}
 		else
 		{
-			x = dsdata_to_cstring(name->value[0]);
+			x = name->value[0];
 		}
 
-		plen += strlen(x);
+		escaped = escape_rdn(x);
+
+		plen += strlen(escaped);
 		p = malloc(plen + 1);
-		sprintf(p, "%s%s", path, x);
+		sprintf(p, "%s%s", path, escaped);
 		free(path);
 		path = p;
 
+		free(escaped);
 		dsdata_release(rdn_key);
 		dsattribute_release(name);
 
@@ -1443,8 +1568,10 @@ dsengine_convert_name(dsengine *e, dsdata *name, u_int32_t intype, u_int32_t des
 	switch (name->type)
 	{
 		case DataTypeCStr:
+		case DataTypeCaseCStr:
 		case DataTypeUTF8Str:
-			tname = dsdata_to_cstring(name);
+		case DataTypeCaseUTF8Str:
+			tname = dsdata_to_utf8string(name);
 			break;
 		case DataTypeDirectoryID:
 			match = dsdata_to_dsid(name);
@@ -1523,10 +1650,12 @@ dsengine_convert_name(dsengine *e, dsdata *name, u_int32_t intype, u_int32_t des
 					if (find_user(e, intype, name, &match) != DSStatusOK)
 						return NULL;
 					str = dsengine_netinfo_string_path(e, match);
+					break;
 				case NameTypeX500:
 					if (find_user(e, intype, name, &match) != DSStatusOK)
 						return NULL;
 					str = dsengine_x500_string_path(e, match);
+					break;
 				case NameTypeDirectoryID:
 					if (find_user(e, intype, name, &match) != DSStatusOK)
 						return NULL;
@@ -1535,6 +1664,7 @@ dsengine_convert_name(dsengine *e, dsdata *name, u_int32_t intype, u_int32_t des
 				default:
 					break;
 			}
+			break;
 		case NameTypeDirectoryID:
 			switch (desiredtype)
 			{
@@ -1556,11 +1686,25 @@ dsengine_convert_name(dsengine *e, dsdata *name, u_int32_t intype, u_int32_t des
 				default:
 					break;
 			}
+			break;
 		default:
 			break;
 	}
 
-	if (str != NULL) d = cstring_to_dsdata(str);
+	if (str != NULL) 
+	{
+		if (d != NULL) dsdata_release(d);
+		d = cstring_to_dsdata(str);
+		free(str);
+	}
 
 	return d;
 }
+
+void dsengine_set_filter_test_delegate(dsengine *e, Logic3 (*delegate)(dsfilter *, dsrecord *, void *), void *private)
+{
+	if (e == NULL) return;
+	e->private = private;
+	e->delegate = delegate;
+}
+
