@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -175,7 +174,7 @@ postfix(char *s, char c)
 	t[len] = '\0';
 	return t;
 }
-	
+
 char *
 presuffix(char *s, char c)
 {
@@ -273,18 +272,10 @@ explode(char *s, char *delim)
 		if (p[i + 1] == '\0') l = appendString("", l);
 		p = p + i + 1;
 	}
+
 	return l;
 }
 
-char *
-itoa(int n)
-{
-	char s[64];
-
-	sprintf(s, "%d", n);
-	return copyString(s);
-}
-
 dsrecord *
 dsutil_parse_netinfo_string_path(char *path)
 {
@@ -293,11 +284,11 @@ dsutil_parse_netinfo_string_path(char *path)
 	u_int32_t i, n, m;
 	dsdata *k, *v;
 	dsattribute *a;
-	
+
 	if (path == NULL) return NULL;
-	
+
 	p = dsrecord_new();
-	
+
 	c = path;
 
 	/* Skip leading slashes */
@@ -579,7 +570,7 @@ dsattribute_from_cstrings(char *key, ...)
 	}
 
 	va_end(ap);
-	
+
 	return a;
 }
 
