@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -27,6 +26,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <time.h>
 #include <netdb.h>
 #include <stdarg.h>
 #include <sys/stat.h>
@@ -38,44 +38,406 @@
 #include <netinet/in.h>
 #include <arpa/nameser.h>
 #include <resolv.h>
+#include <fcntl.h>
 #include <notify.h>
+#include <dnsinfo.h>
 #include "dns_private.h"
 #include "res_private.h"
 
+#define INET_NTOP_AF_INET_OFFSET 4
+#define INET_NTOP_AF_INET6_OFFSET 8
+
 #define DNS_RESOLVER_DIR "/etc/resolver"
 
-#define RESOLV1_NOTIFY_NAME "com.apple.system.dns.resolv.conf"
-#define RESOLV2_NOTIFY_NAME "com.apple.system.dns.resolver"
+#define NOTIFY_DIR_NAME "com.apple.system.dns.resolver.dir"
+#define DNS_DELAY_NAME "com.apple.system.dns.delay"
 
 extern uint32_t notify_monitor_file(int token, const char *name, int flags);
 
-static uint32_t resolv1_token = -1;
-static uint32_t resolv2_token = -1;
+/* notification for sys config changes via dns_configuration_notify_key() */
+static uint32_t notify_resolver_sys_config_token = -1;
+
+/* notify_resolver_dir_token monitors /etc/resolvers */
+static uint32_t notify_resolver_dir_token = -1;
+
+/* notify_resolver_dir_token monitors DNS delay for pppd */
+static uint32_t notify_resolver_delay_token = -1;
+static time_t dns_delay = 0;
+#define DNS_DELAY_INTERVAL 4
 
 #define DNS_PRIVATE_HANDLE_TYPE_SUPER 0
 #define DNS_PRIVATE_HANDLE_TYPE_PLAIN 1
-#define DNS_DEFAULT_RECEIVE_SIZE 1024
-
-#define SDNS_DEFAULT_STAT_LATENCY 10
 
 extern void res_client_close(res_state res);
-extern res_state res_client_open(char *path);
+extern res_state res_state_new();
 extern int res_nquery_2(res_state statp, const char *name, int class, int type, u_char *answer, int anslen, struct sockaddr *from, int *fromlen);
 extern int res_nsearch_2(res_state statp, const char *name, int class, int type, u_char *answer, int anslen, struct sockaddr *from, int *fromlen);
+extern int __res_nsearch_list_2(res_state statp, const char *name,	int class, int type,  u_char *answer, int anslen, struct sockaddr *from, int *fromlen, int nsearch, char **search);
+
+extern char *res_next_word(char **p);
+extern res_state res_build_start(res_state res);
+extern int res_build(res_state res, uint16_t port, uint32_t *nsrch, char *key, char *val);
+extern int res_build_sortlist(res_state res, struct in_addr addr, struct in_addr mask);
+
+static void
+_pdns_set_name(pdns_handle_t *pdns, const char *name)
+{
+	int n;
+
+	if (pdns == NULL) return;
+	if (name == NULL) return;
+
+	/* only set the name once */
+	if (pdns->name != NULL) return;
+
+	/* strip trailing dots */
+	n = strlen(name) - 1;
+	while ((n >= 0) && (name[n] == '.')) n--;
+
+	if (n < 0) return;
+
+	n++;
+	pdns->name = calloc(n + 1, sizeof(char));
+	if (pdns->name == NULL) return;
+	memcpy(pdns->name, name, n);
+}
 
-/*
- * Open a named resolver client.
- */
 static pdns_handle_t *
-_pdns_open(const char *name)
+_pdns_build_start(char *name)
 {
 	pdns_handle_t *pdns;
-	char *path;
-	int status;
-	struct stat sb;
-	struct timeval now;
 
-	memset(&sb, 0, sizeof(struct stat));
+	pdns = (pdns_handle_t *)calloc(1, sizeof(pdns_handle_t));
+	if (pdns == NULL) return NULL;
+
+	pdns->res = res_build_start(NULL);
+	if (pdns->res == NULL)
+	{
+		free(pdns);
+		return NULL;
+	}
+
+	_pdns_set_name(pdns, name);
+	pdns->port = NS_DEFAULTPORT;
+
+	return pdns;
+}
+
+static int
+_pdns_build_finish(pdns_handle_t *pdns)
+{
+	uint32_t n;
+
+	if (pdns == NULL) return -1;
+
+	n = pdns->res->nscount;
+	if (n == 0) n = 1;
+
+	if (pdns->total_timeout == 0)
+	{
+		if (pdns->send_timeout == 0) pdns->total_timeout = RES_MAXRETRANS;
+		else pdns->total_timeout = pdns->send_timeout * pdns->res->retry * n;
+	}
+
+	if (pdns->total_timeout == 0) pdns->res->retrans = RES_MAXRETRANS;
+	else pdns->res->retrans = pdns->total_timeout;
+	
+	pdns->res->options |= RES_INIT;
+
+	return 0;
+}
+
+static int
+_pdns_build_sortlist(pdns_handle_t *pdns, struct in_addr addr, struct in_addr mask)
+{
+	if (pdns == NULL) return -1;
+	return res_build_sortlist(pdns->res, addr, mask);
+}
+
+static int
+_pdns_build(pdns_handle_t *pdns, char *key, char *val)
+{				
+	struct in6_addr addr6;
+	int32_t status;
+
+	if (pdns == NULL) return -1;	
+
+	if (((pdns->flags &= DNS_FLAG_HAVE_IPV6_SERVER) == 0) && (!strcmp(key, "nameserver")))
+	{
+		memset(&addr6, 0, sizeof(struct in6_addr));
+		status = inet_pton(AF_INET6, val, &addr6);
+		if (status == 1) pdns->flags |= DNS_FLAG_HAVE_IPV6_SERVER;
+	}
+
+	if (!strcmp(key, "port"))
+	{
+		pdns->port = atoi(val);
+		return 0;
+	}
+
+	else if (!strcmp(key, "search"))
+	{
+		if (pdns->search_count == 0)
+		{
+			pdns->search_list = (char **)calloc(1, sizeof(char *));
+		}
+		else
+		{
+			pdns->search_list = (char **)realloc(pdns->search_list, (pdns->search_count + 1) * sizeof(char *));
+		}
+	
+		pdns->search_list[pdns->search_count] = strdup(val);
+		pdns->search_count++;
+		return 0;
+	}
+	
+	else if (!strcmp(key, "total_timeout"))
+	{
+		pdns->total_timeout = atoi(val);
+		return 0;
+	}
+	
+	else if (!strcmp(key, "timeout"))
+	{
+		pdns->send_timeout = atoi(val);
+		return 0;
+	}
+	
+	else if (!strcmp(key, "search_order"))
+	{
+		pdns->search_order = atoi(val);
+		return 0;
+	}
+
+	return res_build(pdns->res, pdns->port, &(pdns->search_count), key, val);
+}
+
+static pdns_handle_t *
+_pdns_convert_sc(dns_resolver_t *r)
+{
+	pdns_handle_t *pdns;
+	char *val, *p, *x;
+	int i;
+
+	pdns = _pdns_build_start(r->domain);
+
+	p = getenv("RES_RETRY_TIMEOUT");
+	if (p != NULL) pdns->send_timeout = atoi(p);
+
+	p = getenv("RES_RETRY");
+	if (p != NULL) pdns->res->retry= atoi(p);
+
+	if (r->port != 0)
+	{
+		asprintf(&val, "%hu", r->port);
+		_pdns_build(pdns, "port", val);
+		free(val);
+	}
+
+	if (r->n_nameserver > MAXNS) r->n_nameserver = MAXNS;
+	for (i = 0; i < r->n_nameserver; i++)
+	{
+		if (r->nameserver[i]->sa_family == AF_INET)
+		{
+			val = calloc(1, 256);
+			inet_ntop(AF_INET, (char *)(r->nameserver[i]) + INET_NTOP_AF_INET_OFFSET, val, 256);
+			_pdns_build(pdns, "nameserver", val);
+			free(val);
+		}
+		else if (r->nameserver[i]->sa_family == AF_INET6)
+		{
+			pdns->flags |= DNS_FLAG_HAVE_IPV6_SERVER;
+			val = calloc(1, 256);
+			inet_ntop(AF_INET6, (char *)(r->nameserver[i]) + INET_NTOP_AF_INET6_OFFSET, val, 256);
+			_pdns_build(pdns, "nameserver", val);
+			free(val);
+		}
+	}
+
+	if (r->n_search > MAXDNSRCH) r->n_search = MAXDNSRCH;
+	for (i = 0; i < r->n_search; i++)
+	{
+		asprintf(&val, "%s", r->search[i]);
+		_pdns_build(pdns, "search", val);
+		free(val);
+	}
+
+	if (r->timeout > 0)
+	{
+		asprintf(&val, "%d", r->timeout);
+		_pdns_build(pdns, "total_timeout", val);
+		free(val);
+	}
+
+	asprintf(&val, "%d", r->search_order);
+	_pdns_build(pdns, "search_order", val);
+	free(val);
+
+	if (r->n_sortaddr > MAXRESOLVSORT) r->n_sortaddr = MAXRESOLVSORT;
+	for (i = 0; i < r->n_sortaddr; i++)
+	{
+		_pdns_build_sortlist(pdns, r->sortaddr[i]->address, r->sortaddr[i]->mask);
+	}
+
+	p = r->options;
+	while (NULL != (x = res_next_word(&p)))
+	{
+		/* search for and process individual options */
+		if (!strncmp(x, "ndots:", 6))
+		{
+			_pdns_build(pdns, "ndots", x+6);
+		}
+
+		else if (!strncmp(x, "nibble:", 7))
+		{
+			_pdns_build(pdns, "nibble", x+7);
+		}
+		
+		else if (!strncmp(x, "nibble2:", 8))
+		{
+			_pdns_build(pdns, "nibble2", x+8);
+		}
+		
+		else if (!strncmp(x, "timeout:", 8))
+		{
+			_pdns_build(pdns, "timeout", x+8);
+		}
+		
+		else if (!strncmp(x, "attempts:", 9))
+		{
+			_pdns_build(pdns, "attempts", x+9);
+		}
+		
+		else if (!strncmp(x, "bitstring:", 10))
+		{
+			_pdns_build(pdns, "bitstring", x+10);
+		}
+		
+		else if (!strncmp(x, "v6revmode:", 10))
+		{
+			_pdns_build(pdns, "v6revmode", x+10);
+		}
+		
+		else if (!strcmp(x, "debug"))
+		{
+			_pdns_build(pdns, "debug", NULL);
+		}
+		
+		else if (!strcmp(x, "no_tld_query"))
+		{
+			_pdns_build(pdns, "no_tld_query", NULL);
+		}
+
+		else if (!strcmp(x, "inet6"))
+		{
+			_pdns_build(pdns, "inet6", NULL);
+		}
+		
+		else if (!strcmp(x, "rotate"))
+		{
+			_pdns_build(pdns, "rotate", NULL);
+		}
+
+		else if (!strcmp(x, "no-check-names"))
+		{
+			_pdns_build(pdns, "no-check-names", NULL);
+		}
+		
+#ifdef RES_USE_EDNS0
+		else if (!strcmp(x, "edns0"))
+		{
+			_pdns_build(pdns, "edns0", NULL);
+		}		
+#endif
+		else if (!strcmp(x, "a6"))
+		{
+			_pdns_build(pdns, "a6", NULL);
+		}
+		
+		else if (!strcmp(x, "dname"))
+		{
+			_pdns_build(pdns, "dname", NULL);
+		}
+	}
+
+	_pdns_build_finish(pdns);
+	return pdns;
+}
+
+/*
+ * Open a named resolver client from the system config data.
+ */
+static pdns_handle_t *
+_pdns_sc_open(const char *name)
+{
+	pdns_handle_t *pdns;
+	int i;
+	dns_config_t *sc_dns;
+	dns_resolver_t *sc_res;
+
+	sc_dns = dns_configuration_copy();
+	if (sc_dns == NULL) return NULL;
+
+	sc_res = NULL;
+
+	if (name == NULL)
+	{
+		if (sc_dns->n_resolver != 0) sc_res = sc_dns->resolver[0];
+	}
+	else
+	{
+		for (i = 0; (sc_res == NULL) && (i < sc_dns->n_resolver); i++)
+		{
+			if (sc_dns->resolver[i] == NULL) continue;
+			if (sc_dns->resolver[i]->domain == NULL) continue;
+			if (!strcasecmp(name, sc_dns->resolver[i]->domain)) sc_res = sc_dns->resolver[i];
+		}
+	}
+
+	if (sc_res == NULL)
+	{
+		dns_configuration_free(sc_dns);
+		return NULL;
+	}
+
+	pdns = (pdns_handle_t *)calloc(1, sizeof(pdns_handle_t));
+	if (pdns == NULL)
+	{
+		dns_configuration_free(sc_dns);
+		return NULL;
+	}
+
+	pdns = _pdns_convert_sc(sc_res);
+	
+	dns_configuration_free(sc_dns);
+
+	if (pdns == NULL) return NULL;
+
+	if (pdns->res == NULL)
+	{
+		free(pdns);
+		return NULL;
+	}
+
+	pdns->name = NULL;
+	if (pdns->res->defdname[0] != '\0') _pdns_set_name(pdns, pdns->res->defdname);
+	else if (name != NULL) _pdns_set_name(pdns, name);
+		
+	if (name != NULL) pdns->search_count = -1;
+
+	return pdns;
+}
+
+/*
+ * Open a named resolver client from file.
+ */
+static pdns_handle_t *
+_pdns_file_open(const char *name)
+{
+	pdns_handle_t *pdns;
+	char *path, buf[1024];
+	char *p, *x, *y;
+	FILE *fp;
 
 	path = NULL;
 	if (name == NULL)
@@ -91,40 +453,69 @@ _pdns_open(const char *name)
 		asprintf(&path, "%s/%s", DNS_RESOLVER_DIR, name);
 	}
 
-	status = stat(path, &sb);
-	if (status < 0)
-	{
-		free(path);
-		return NULL;
-	}
-
-	pdns = (pdns_handle_t *)calloc(1, sizeof(pdns_handle_t));
-	if (pdns == NULL)
-	{
-		free(path);
-		return NULL;
-	}
-
-	pdns->res = res_client_open(path);
+	fp = fopen(path, "r");
+	free(path);
+	if (fp == NULL) return NULL;
 	
-	if (pdns->res == NULL)
+	pdns = _pdns_build_start(NULL);
+	if (pdns == NULL) 
 	{
-		free(path);
-		free(pdns);
+		fclose(fp);
 		return NULL;
 	}
 
-	pdns->source = path;
+	p = getenv("RES_RETRY_TIMEOUT");
+	if (p != NULL) pdns->send_timeout = atoi(p);
 
-	pdns->name = NULL;
-	if (pdns->res->defdname[0] != '\0') pdns->name = strdup(pdns->res->defdname);
-	else if (name != NULL) pdns->name = strdup(name);
+	p = getenv("RES_RETRY");
+	if (p != NULL) pdns->res->retry= atoi(p);
 
-	gettimeofday(&now, NULL);
-	pdns->modtime = sb.st_mtimespec.tv_sec;
-	pdns->stattime = now.tv_sec;
+	while (fgets(buf, sizeof(buf), fp) != NULL)
+	{
+		/* skip comments */
+		if ((buf[0] == ';') || (buf[0] == '#')) continue;
+		p = buf;
+		x = res_next_word(&p);
+		if (x == NULL) continue;
+		if (!strcmp(x, "sortlist"))
+		{
+			while (NULL != (x = res_next_word(&p)))
+			{
+				_pdns_build(pdns, "sortlist", x);
+			}
+		}
+		else if (!strcmp(x, "timeout"))
+		{
+			x = res_next_word(&p);
+			if (x != NULL) _pdns_build(pdns, "total_timeout", x);
+		}
+		else if (!strcmp(x, "options"))
+		{
+			while (NULL != (x = res_next_word(&p)))
+			{
+				y = strchr(x, ':');
+				if (y != NULL)
+				{
+					*y = '\0';
+					y++;
+				}
+				_pdns_build(pdns, x, y);
+			}
+		}
+		else 
+		{
+			y = res_next_word(&p);
+			_pdns_build(pdns, x, y);
 
-	pdns->search_count = -1;
+			if ((!strcmp(x, "domain")) && (pdns->name == NULL)) _pdns_set_name(pdns, y);
+		}
+	}
+
+	fclose(fp);
+
+	if (pdns->name == NULL) _pdns_set_name(pdns, name);
+
+	_pdns_build_finish(pdns);
 
 	return pdns;
 }
@@ -133,63 +524,27 @@ static pdns_handle_t *
 _pdns_copy(pdns_handle_t *in)
 {
 	pdns_handle_t *pdns;
-	char *path;
-	int status;
-	struct stat sb;
-	struct timeval now;
 
 	if (in == NULL) return NULL;
 
-	memset(&sb, 0, sizeof(struct stat));
-
-	path = NULL;
-	if (in->source != NULL) path = strdup(in->source);
-
-	if (path == NULL)
-	{
-		if (in->name == NULL)
-		{
-			asprintf(&path, "%s", _PATH_RESCONF);
-		}
-		else if ((in->name[0] == '.') || (in->name[0] == '/'))
-		{
-			asprintf(&path, "%s", in->name);
-		}
-		else
-		{
-			asprintf(&path, "%s/%s", DNS_RESOLVER_DIR, in->name);
-		}
-	}
-
-	if (path == NULL) return NULL;
-
-	status = stat(path, &sb);
-	if (status < 0)
-	{
-		free(path);
-		return NULL;
-	}
-
 	pdns = (pdns_handle_t *)calloc(1, sizeof(pdns_handle_t));
 	if (pdns == NULL) return NULL;
 
-	pdns->res = res_client_open(path);
-	free(path);
+	memcpy(pdns, in, sizeof(pdns_handle_t));
+
+	pdns->res = res_state_new();
 	if (pdns->res == NULL)
 	{
 		free(pdns);
 		return NULL;
 	}
 
-	pdns->name = NULL;
-	if (in->name != NULL) pdns->name = strdup(in->name);
-	else if (pdns->res->defdname[0] != '\0') pdns->name = strdup(pdns->res->defdname);
-
-	gettimeofday(&now, NULL);
-	pdns->modtime = sb.st_mtimespec.tv_sec;
-	pdns->stattime = now.tv_sec;
-
-	pdns->search_count = -1;
+	memcpy(pdns->res, in->res, sizeof(struct __res_state));
+	/* NB res_state_new() allocates a struct __res_state_ext for _u._ext.ext */
+	if (in->res->_u._ext.ext != NULL) 
+	{
+		memcpy(pdns->res->_u._ext.ext, in->res->_u._ext.ext, sizeof(struct __res_state_ext));
+	}
 
 	return pdns;
 }
@@ -197,91 +552,190 @@ _pdns_copy(pdns_handle_t *in)
 static void
 _pdns_free(pdns_handle_t *pdns)
 {
+	int i;
+
 	if (pdns == NULL) return;
 
+	if ((pdns->search_count != -1) && (pdns->search_count > 0))
+	{
+		for (i = 0; i < pdns->search_count; i++) free(pdns->search_list[i]);
+		free(pdns->search_list);
+	}
+
 	if (pdns->name != NULL) free(pdns->name);
-	if (pdns->source != NULL) free(pdns->source);
 	if (pdns->res != NULL) res_client_close(pdns->res);
 
 	free(pdns);
 }
 
+/*
+ * If there was no search list, use domain name and parent domain components.
+ *
+ * N.B. This code deals with multiple trailing dots, but does not deal with
+ * multiple internal dots, e.g. "foo.....com".  
+ */
+static void
+_pdns_check_search_list(pdns_handle_t *pdns)
+{
+	int n;
+	char *p;
+
+	if (pdns == NULL) return;
+	if (pdns->name == NULL) return;
+	if (pdns->search_count > 0) return;
+
+	/* Count dots */
+	n = 0;
+	for (p = pdns->name; *p != '\0'; p++) 
+	{
+		if (*p == '.') n++;
+	}
+
+	/* Back up over any trailing dots and cut them out of the name */
+	for (p--; (p >= pdns->name) && (*p == '.'); p--)
+	{
+		*p = '\0';
+		n--;
+	}
+
+	/* This will be true if name was all dots */
+	if (p < pdns->name) return;
+
+	/* dots are separators, so number of components is one larger */
+	n++;
+
+	_pdns_build(pdns, "search", pdns->name);
+
+	/* Include parent domains with at least LOCALDOMAINPARTS components */
+	p = pdns->name;
+	while (n > LOCALDOMAINPARTS)
+	{
+		/* Find next component */
+		while ((*p != '.') && (*p != '\0')) p++;
+		if (*p == '\0') break;
+		p++;
+
+		n--;
+		_pdns_build(pdns, "search", p);
+	}
+}
+
 static void
 _check_cache(sdns_handle_t *sdns)
 {
-	struct stat sb;
-	int i, n, status;
+	int i, n, status, refresh, sc_dns_count;
 	DIR *dp;
 	struct direct *d;
 	pdns_handle_t *c;
-	struct timeval now;
+	dns_config_t *sc_dns;
 
 	if (sdns == NULL) return;
 
-	if ((sdns->stattime != 0) && (resolv1_token != -1) && (resolv2_token != -1))
+	refresh = 0;
+
+	if (sdns->stattime == 0) refresh = 1;
+	
+	if (refresh == 0)
 	{
-		n = 1;
-		status = notify_check(resolv1_token, &n);
-		if ((status == NOTIFY_STATUS_OK) && (n == 0))
+		if (notify_resolver_sys_config_token == -1) refresh = 1;
+		else
 		{
 			n = 1;
-			status = notify_check(resolv2_token, &n);
-			if ((status == NOTIFY_STATUS_OK) && (n == 0)) return;
+			status = notify_check(notify_resolver_sys_config_token, &n);
+			if ((status != NOTIFY_STATUS_OK) || (n == 1)) refresh = 1;
 		}
 	}
 
-	gettimeofday(&now, NULL);
+	if (refresh == 0)
+	{
+		if (notify_resolver_dir_token == -1) refresh = 1;
+		else
+		{
+			n = 1;
+			status = notify_check(notify_resolver_dir_token, &n);
+			if ((status != NOTIFY_STATUS_OK) || (n == 1)) refresh = 1;
+		}
+	}
+	
+	if (refresh == 0) return;
+
+	/* Free old clients */
+	_pdns_free(sdns->dns_default);
+	sdns->dns_default = NULL;
+
+	for (i = 0; i < sdns->client_count; i++)
+	{
+		_pdns_free(sdns->client[i]);
+	}
+	
+	sdns->client_count = 0;
+	if (sdns->client != NULL) free(sdns->client);
+	sdns->client = NULL;
+
+	/* Fetch clients from System Configuration */
+	sc_dns = dns_configuration_copy();
+
+	sc_dns_count = 0;
+	if ((sc_dns != NULL) && (sc_dns->n_resolver > 0))
+	{
+		sc_dns_count = sc_dns->n_resolver;
+		sdns->dns_default = _pdns_convert_sc(sc_dns->resolver[0]);
+		_pdns_check_search_list(sdns->dns_default);
+	}
+	else
+	{
+		sdns->dns_default = _pdns_file_open(_PATH_RESCONF);
+	}
 
 	if (sdns->dns_default != NULL)
 	{
-		/* Check default (/etc/resolv.conf) client */
-		memset(&sb, 0, sizeof(struct stat));
-		status = stat(_PATH_RESCONF, &sb);
-		if ((status != 0) || (sb.st_mtimespec.tv_sec > sdns->dns_default->modtime))
-		{
-			_pdns_free(sdns->dns_default);
-			sdns->dns_default = NULL;
-		}
-		else if (status == 0)
-		{
-			sdns->dns_default->stattime = now.tv_sec;
-		}
-	}
-	
-	if (sdns->dns_default == NULL)
-	{
-		sdns->dns_default = _pdns_open(NULL);
-		if ((sdns->dns_default != NULL) && (sdns->flags & DNS_FLAG_DEBUG)) sdns->dns_default->res->options |= RES_DEBUG;
-	}
-	
-	/* Check /etc/resolver clients */
-	memset(&sb, 0, sizeof(struct stat));
-	status = stat(DNS_RESOLVER_DIR, &sb);
-	if ((status != 0) || (sb.st_mtimespec.tv_sec > sdns->modtime))
-	{
-		for (i = 0; i < sdns->client_count; i++)
-		{
-			_pdns_free(sdns->client[i]);
-		}
-
-		sdns->client_count = 0;
-		if (sdns->client != NULL) free(sdns->client);
-		sdns->client = NULL;
+		if (sdns->flags & DNS_FLAG_DEBUG) sdns->dns_default->res->options |= RES_DEBUG;
+		if (sdns->flags & DNS_FLAG_OK_TO_SKIP_AAAA) sdns->dns_default->flags |= DNS_FLAG_OK_TO_SKIP_AAAA;
 	}
 
-	if ((status == 0) && (sb.st_mtimespec.tv_sec > sdns->modtime))
+	/* Convert System Configuration resolvers */
+	for (i = 1; i < sc_dns_count; i++)
 	{
+		c = _pdns_convert_sc(sc_dns->resolver[i]);
+		if (c == NULL) continue;
+
+		if (sdns->flags & DNS_FLAG_DEBUG) c->res->options |= RES_DEBUG;
+		if (sdns->flags & DNS_FLAG_OK_TO_SKIP_AAAA) c->flags |= DNS_FLAG_OK_TO_SKIP_AAAA;
+
+		if (sdns->client_count == 0)
+		{
+			sdns->client = (pdns_handle_t **)malloc(sizeof(pdns_handle_t *));
+		}
+		else
+		{
+			sdns->client = (pdns_handle_t **)realloc(sdns->client, (sdns->client_count + 1) * sizeof(pdns_handle_t *));
+		}
+	
+		sdns->client[sdns->client_count] = c;
+		sdns->client_count++;
+	}
+
+	if (sc_dns != NULL) dns_configuration_free(sc_dns);
+
+	if (sdns->flags & DNS_FLAG_CHECK_RESOLVER_DIR)
+	{
+		/* Read /etc/resolvers clients */
 		dp = opendir(DNS_RESOLVER_DIR);
-		if (dp != NULL)
+		if (dp == NULL)
+		{
+			sdns->flags &= ~DNS_FLAG_CHECK_RESOLVER_DIR;
+		}
+		else
 		{
 			while (NULL != (d = readdir(dp)))
 			{
 				if (d->d_name[0] == '.') continue;
-
-				c = _pdns_open(d->d_name);
+				
+				c = _pdns_file_open(d->d_name);
 				if (c == NULL) continue;
 				if (sdns->flags & DNS_FLAG_DEBUG) c->res->options |= RES_DEBUG;
-
+				if (sdns->flags & DNS_FLAG_OK_TO_SKIP_AAAA) c->flags |= DNS_FLAG_OK_TO_SKIP_AAAA;
+				
 				if (sdns->client_count == 0)
 				{
 					sdns->client = (pdns_handle_t **)malloc(sizeof(pdns_handle_t *));
@@ -290,20 +744,15 @@ _check_cache(sdns_handle_t *sdns)
 				{
 					sdns->client = (pdns_handle_t **)realloc(sdns->client, (sdns->client_count + 1) * sizeof(pdns_handle_t *));
 				}
-			
+				
 				sdns->client[sdns->client_count] = c;
 				sdns->client_count++;
 			}
 			closedir(dp);
 		}
-			
-		sdns->modtime = sb.st_mtimespec.tv_sec;
-		sdns->stattime = now.tv_sec;
-	}
-	else if (status == 0)
-	{
-		sdns->stattime = now.tv_sec;
 	}
+	
+	sdns->stattime = 1;
 }
 
 static uint32_t
@@ -357,7 +806,7 @@ _pdns_get_handles_for_name(sdns_handle_t *sdns, const char *name, pdns_handle_t
 				/* Insert sorted by search_order */
 				for (j = 0; j < count; j++)
 				{
-					if (sdns->client[i]->res->_u._ext.ext->search_order < (*pdns)[j]->res->_u._ext.ext->search_order) break;
+					if (sdns->client[i]->search_order < (*pdns)[j]->search_order) break;
 				}
 
 				for (k = count; k > j; k--) (*pdns)[k] = (*pdns)[k-1];
@@ -374,14 +823,6 @@ _pdns_get_handles_for_name(sdns_handle_t *sdns, const char *name, pdns_handle_t
 
 	if (count != 0)  return count;
 
-	if (sdns->dns_default == NULL)
-	{
-		sdns->dns_default = _pdns_open(NULL);
-		if (sdns->dns_default == NULL) return 0;
-
-		if (sdns->flags & DNS_FLAG_DEBUG) sdns->dns_default->res->options |= RES_DEBUG;
-	}
-
 	if (sdns->dns_default == NULL) return 0;
 
 	*pdns = (pdns_handle_t **)calloc(1, sizeof(pdns_handle_t *));
@@ -406,15 +847,6 @@ _pdns_process_res_search_list(pdns_handle_t *pdns)
 	for (pdns->search_count = 0; (pdns->res->dnsrch[pdns->search_count] != NULL) && (pdns->res->dnsrch[pdns->search_count][0] != '\0'); pdns->search_count++);
 }
 
-static uint32_t
-_pdns_search_list_domain_count(pdns_handle_t *pdns)
-{
-	if (pdns == NULL) return 0;
-	
-	_pdns_process_res_search_list(pdns);
-	return (pdns->search_count);
-}
-
 static char *
 _pdns_search_list_domain(pdns_handle_t *pdns, uint32_t i)
 {
@@ -422,10 +854,9 @@ _pdns_search_list_domain(pdns_handle_t *pdns, uint32_t i)
 
 	if (pdns == NULL) return NULL;
 	
-	_pdns_process_res_search_list(pdns);
 	if (i >= pdns->search_count) return NULL;
 
-	s = pdns->res->dnsrch[i];
+	s = pdns->search_list[i];
 	if (s == NULL) return NULL;
 	return strdup(s);
 }
@@ -433,45 +864,36 @@ _pdns_search_list_domain(pdns_handle_t *pdns, uint32_t i)
 void
 __dns_open_notify()
 {
-	uint32_t status;
+	uint32_t status, n;
 
-	if (resolv1_token == -1)
+	if (notify_resolver_delay_token == -1)
 	{
-		status = notify_register_check(RESOLV1_NOTIFY_NAME, &resolv1_token);
-		if (status == NOTIFY_STATUS_OK)
-		{
-			status = notify_monitor_file(resolv1_token, "/var/run/resolv.conf", 0);
-			if (status != NOTIFY_STATUS_OK)
-			{
-				notify_cancel(resolv1_token);
-				resolv1_token = -1;
-			}
-		}
-		else 
-		{
-			resolv1_token = -1;
-		}
+		status = notify_register_check(DNS_DELAY_NAME, &notify_resolver_delay_token);
+		if (status != NOTIFY_STATUS_OK) notify_resolver_delay_token = -1;
+		else status = notify_check(notify_resolver_delay_token, &n);
 	}
 
-	if ((resolv1_token != -1 ) && (resolv2_token == -1))
+	if (notify_resolver_sys_config_token == -1)
 	{
-		status = notify_register_check(RESOLV2_NOTIFY_NAME, &resolv2_token);
+		status = notify_register_check(dns_configuration_notify_key(), &notify_resolver_sys_config_token);
+		if (status != NOTIFY_STATUS_OK) notify_resolver_sys_config_token = -1;
+	}
+
+	if (notify_resolver_dir_token == -1)
+	{
+		status = notify_register_check(NOTIFY_DIR_NAME, &notify_resolver_dir_token);
 		if (status == NOTIFY_STATUS_OK)
 		{
-			status = notify_monitor_file(resolv2_token, "/private/etc/resolver", 0);
+			status = notify_monitor_file(notify_resolver_dir_token, "/private/etc/resolver", 0);
 			if (status != NOTIFY_STATUS_OK)
 			{
-				notify_cancel(resolv1_token);
-				notify_cancel(resolv2_token);
-				resolv1_token = -1;
-				resolv2_token = -1;
+				notify_cancel(notify_resolver_dir_token);
+				notify_resolver_dir_token = -1;
 			}
 		}
 		else 
 		{
-			notify_cancel(resolv1_token);
-			resolv1_token = -1;
-			resolv2_token = -1;
+			notify_resolver_dir_token = -1;
 		}
 	}
 }
@@ -479,11 +901,11 @@ __dns_open_notify()
 void
 __dns_close_notify()
 {
-	if (resolv1_token != -1) notify_cancel(resolv1_token);
-	resolv1_token = -1;
+	if (notify_resolver_sys_config_token != -1) notify_cancel(notify_resolver_sys_config_token);
+	notify_resolver_sys_config_token = -1;
 
-	if (resolv2_token != -1) notify_cancel(resolv2_token);
-	resolv2_token = -1;
+	if (notify_resolver_dir_token != -1) notify_cancel(notify_resolver_dir_token);
+	notify_resolver_dir_token = -1;
 }
 
 dns_handle_t
@@ -504,12 +926,17 @@ dns_open(const char *name)
 			return NULL;
 		}
 
-		dns->sdns->stat_latency = SDNS_DEFAULT_STAT_LATENCY;
+		dns->sdns->flags |= DNS_FLAG_CHECK_RESOLVER_DIR;
+
 		return (dns_handle_t)dns;
 	}
 
 	dns->handle_type = DNS_PRIVATE_HANDLE_TYPE_PLAIN;
-	dns->pdns = _pdns_open(name);
+
+	/*  Look for name in System Configuration first */
+	dns->pdns = _pdns_sc_open(name);
+	if (dns->pdns == NULL) dns->pdns = _pdns_file_open(name);
+
 	if (dns->pdns == NULL)
 	{
 		free(dns);
@@ -654,7 +1081,7 @@ dns_search_list_count(dns_handle_t d)
 		pdns = dns->pdns;
 	}
 
-	return _pdns_search_list_domain_count(pdns);
+	return pdns->search_count;
 }
 
 /* 
@@ -684,12 +1111,64 @@ dns_search_list_domain(dns_handle_t d, uint32_t i)
 	return _pdns_search_list_domain(pdns, i);
 }
 
+static int
+_pdns_delay()
+{
+	int status, n, snooze;
+	time_t tick;
+
+	snooze = 0;
+	n = 0;
+
+	/* No delay if we are not receiving notifications */
+	if (notify_resolver_delay_token == -1) return 0;
+
+	if (dns_delay == 0)
+	{
+		status = notify_check(notify_resolver_delay_token, &n);
+		if ((status == NOTIFY_STATUS_OK) && (n == 1))
+		{
+			/*
+			 * First thread to hit this condition sleeps for DNS_DELAY_INTERVAL seconds
+			 */
+			dns_delay = time(NULL) + DNS_DELAY_INTERVAL;
+			snooze = DNS_DELAY_INTERVAL;
+		}
+	}
+	else
+	{
+		tick = time(NULL);
+		/*
+		 * Subsequent threads sleep for the remaining duration.
+		 * We add one to round up the interval since our garnularity is coarse.
+		 */
+		snooze = 1 + (dns_delay - tick);
+		if (snooze < 0) snooze = 0;
+	}
+
+	if (snooze == 0) return 0;
+
+	sleep(snooze);
+
+	/* When exiting, first thread in resets the delay condition */
+	if (n == 1) dns_delay = 0;
+
+	return 0;
+}
+
 static int
 _pdns_query(pdns_handle_t *pdns, const char *name, uint32_t class, uint32_t type, char *buf, uint32_t len, struct sockaddr *from, int *fromlen)
 {
+	if (name == NULL) return -1;
 	if (pdns == NULL) return -1;
 	if (pdns->res == NULL) return -1;
+	if (pdns->res->nscount == 0) return -1;
 
+	if ((type == ns_t_aaaa) && ((pdns->flags & DNS_FLAG_HAVE_IPV6_SERVER) == 0) && (pdns->flags & DNS_FLAG_OK_TO_SKIP_AAAA)) return -1;
+
+	_pdns_delay();
+
+	/* BIND_9 API */
 	return res_nquery_2(pdns->res, name, class, type, buf, len, from, fromlen);
 }
 
@@ -697,10 +1176,14 @@ static int
 _pdns_search(pdns_handle_t *pdns, const char *name, uint32_t class, uint32_t type, char *buf, uint32_t len, struct sockaddr *from, int *fromlen)
 {
 	char *dot, *qname;
-	uint32_t append, status;
+	int append, status;
 
-	if (pdns->res == NULL) return -1;
 	if (name == NULL) return -1;
+	if (pdns == NULL) return -1;
+	if (pdns->res == NULL) return -1;
+	if (pdns->res->nscount == 0) return -1;
+
+	if ((type == ns_t_aaaa) && ((pdns->flags & DNS_FLAG_HAVE_IPV6_SERVER) == 0) && (pdns->flags & DNS_FLAG_OK_TO_SKIP_AAAA)) return -1;
 
 	qname = NULL;
 	append = 1;
@@ -732,11 +1215,17 @@ _pdns_search(pdns_handle_t *pdns, const char *name, uint32_t class, uint32_t typ
 	status = -1;
 	if (append == 0)
 	{
-		status = res_nsearch_2(pdns->res, name, class, type, buf, len, from, fromlen);
+		/* BIND_9 API */
+		_pdns_delay();
+
+		status = __res_nsearch_list_2(pdns->res, name, class, type, buf, len, from, fromlen, pdns->search_count, pdns->search_list);
 	}
 	else
 	{
+		_pdns_delay();
+
 		asprintf(&qname, "%s.%s.", name, pdns->name);
+		/* BIND_9 API */
 		status = res_nsearch_2(pdns->res, qname, class, type, buf, len, from, fromlen);
 		free(qname);
 	}
@@ -778,8 +1267,8 @@ static int
 _sdns_search(sdns_handle_t *sdns, const char *name, uint32_t class, uint32_t type, uint32_t fqdn, uint32_t recurse, char *buf, uint32_t len, struct sockaddr *from, uint32_t *fromlen)
 {
 	pdns_handle_t *pdns;
-	int i, n, status;
-	char *dot, *s, *qname;
+	int i, j, n, ndots, status;
+	char *dot, *qname, **search_domain;
 
 	/*
 	 * If name is qualified:
@@ -793,15 +1282,29 @@ _sdns_search(sdns_handle_t *sdns, const char *name, uint32_t class, uint32_t typ
 	 *         call sdns_query with the name qualified with default domain name.
 	 */
 
-	if (sdns == NULL) return NULL;
-	if (name == NULL) return NULL;
+	if (sdns == NULL) return -1;
+	if (name == NULL) return -1;
 
-	dot = strrchr(name, '.');
+	ndots = 1;
+	pdns = _pdns_default_handle(sdns);
+	if ((pdns != NULL) && (pdns->res != NULL)) ndots = pdns->res->ndots;
+
+	n = 0;
+	dot = NULL;
+
+	for (i = 0; name[i] != '\0'; i++)
+	{
+		if (name[i] == '.')
+		{
+			n++;
+			dot = (char *)(name + i);
+		}
+	}
 
 	if ((fqdn == 0 ) && (dot != NULL) && (*(dot + 1) == '\0')) fqdn = 1;
 
-	/* If name is qualified: */
-	if (dot != NULL)
+	/* If name is partly qualified: */
+	if (n >= ndots)
 	{
 		status = _sdns_send(sdns, name, class, type, fqdn, buf, len, from, fromlen);
 		if (status > 0) return status;
@@ -813,21 +1316,31 @@ _sdns_search(sdns_handle_t *sdns, const char *name, uint32_t class, uint32_t typ
 	pdns = _pdns_default_handle(sdns);
 	if (pdns == NULL) return -1;
 
-	n = _pdns_search_list_domain_count(pdns);
+	n = pdns->search_count;
 	if (n > 0)
 	{
+		/* We need to copy the search list since _sdns_search() can change the cache */
+		search_domain = calloc(n, sizeof(char *));
+		if (search_domain == NULL) return -1;
+
+		for (i = 0; i < n ; i++) search_domain[i] = strdup(pdns->search_list[i]);
+
 		for (i = 0; i < n ; i++)
 		{
-			s = _pdns_search_list_domain(pdns, i);
-			if (s == NULL) continue;
-
-			asprintf(&qname, "%s.%s", name, s);
-			free(s);
+			asprintf(&qname, "%s.%s", name, search_domain[i]);
 			status = _sdns_search(sdns, qname, class, type, fqdn, 0, buf, len, from, fromlen);
 			free(qname);
-			if (status > 0) return status;
+			if (status > 0)
+			{
+				for (j = 0; j < n ; j++) free(search_domain[j]);
+				free(search_domain);
+				return status;
+			}
 		}
 
+		for (i = 0; i < n ; i++) free(search_domain[i]);
+		free(search_domain);
+
 		return -1;
 	}
 	
@@ -910,6 +1423,7 @@ dns_clients_for_name(dns_handle_t d, const char *name)
 
 	for (i = 0; i < count; i++)
 	{
+		c[i] = (dns_private_handle_t *)calloc(1, sizeof(dns_private_handle_t));
 		c[i]->handle_type = DNS_PRIVATE_HANDLE_TYPE_PLAIN;
 		c[i]->pdns = _pdns_copy(x[i]);
 	}
