@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -43,13 +42,16 @@
 #import <sys/dir.h>
 #import <arpa/inet.h>
 
+extern ni_shared_handle_t *ni_raw_local(void);
+extern void ni_release_raw_local(void);
+
 @implementation Config
 
 - (char *)configDirNameForAgent:(char *)agent category:(LUCategory)cat
 {
 	char str[256];
 	char catname[64];
-	
+
 	if (cat == LUCategoryNull) strcpy(catname, "Global");
 	else 
 	{
@@ -62,7 +64,7 @@
 		sprintf(str, "%s Configuration", catname);
 		return copyString(str);
 	}
-	
+
 	if (cat == LUCategoryNull) sprintf(str, "%s Configuration", agent);
 	else sprintf(str, "%s %s Configuration", agent, catname);
 	return copyString(str);
@@ -163,7 +165,7 @@
 	[cdict release];
 	if (sourcePath != NULL) free(sourcePath);
 	if (sourceDomainName != NULL) free(sourceDomainName);
-	
+
 	[super dealloc];
 }
 
@@ -252,7 +254,6 @@
 
 	if (status != NI_OK)
 	{
-		ni_shared_release(sourceDomain);
 		sourceDomain = NULL;
 		return;
 	}
@@ -313,7 +314,7 @@
 	if (sourceDomain == NULL) return NO;
 	if (sourcePath == NULL)
 	{
-		ni_shared_release(sourceDomain);
+		if (sourceDomainIsRawLocal) ni_release_raw_local();
 		sourceDomain = NULL;
 		return NO;
 	}
@@ -325,7 +326,7 @@
 
 	if (status != NI_OK)
 	{
-		ni_shared_release(sourceDomain);
+		if (sourceDomainIsRawLocal) ni_release_raw_local();
 		sourceDomain = NULL;
 		return NO;
 	}
@@ -354,7 +355,7 @@
 
 	if (status != NI_OK)
 	{
-		ni_shared_release(sourceDomain);
+		if (sourceDomainIsRawLocal) ni_release_raw_local();
 		sourceDomain = NULL;
 		if (status == NI_NODIR) return YES;
 		return NO;
@@ -368,7 +369,7 @@
 
 	if (status != NI_OK)
 	{
-		ni_shared_release(sourceDomain);
+		if (sourceDomainIsRawLocal) ni_release_raw_local();
 		sourceDomain = NULL;
 		return YES;
 	}
@@ -384,7 +385,7 @@
 
 	ni_entrylist_free(&el);
 
-	if (sourceDomain != NULL) ni_shared_release(sourceDomain);
+	if (sourceDomainIsRawLocal) ni_release_raw_local();
 	sourceDomain = NULL;
 
 	return YES;
@@ -454,12 +455,12 @@
 				if (p[0] == sep[j] || (p[0] == '\0')) scanning = NO;
 			}
 		}
-	
+
 		/* back over trailing whitespace */
 		i--;
 		while ((buf[i] == ' ') || (buf[i] == '\t') || (buf[i] == '\n')) i--;
 		buf[++i] = '\0';
-	
+
 		tokens = appendString(buf, tokens);
 
 		/* check for end of line */
@@ -632,7 +633,6 @@
 	source = configSourceDefault;
 	if (sourcePath != NULL) freeString(sourcePath);
 	sourcePath = NULL;
-	if (sourceDomain != NULL) ni_shared_release(sourceDomain);
 	sourceDomain = NULL;
 
 	/* Check file:/etc/lookupd */
@@ -646,9 +646,12 @@
 		}
 	}
 
-	/* Check netinfo:/config/lookupd */
 	syslock_lock(rpcLock);
-	status = sa_find(&sourceDomain, &nid, "/config/lookupd", 30);
+	sourceDomain = ni_raw_local();
+	sourceDomainIsRawLocal = YES;
+	
+	/* Check netinfo:/config/lookupd */
+	status = sa_pathsearch(sourceDomain, &nid, "/config/lookupd");
 	syslock_unlock(rpcLock);
 	if (status == NI_OK)
 	{
@@ -659,7 +662,7 @@
 
 	/* Check netinfo:/locations/lookupd */
 	syslock_lock(rpcLock);
-	status = sa_find(&sourceDomain, &nid, "/locations/lookupd", 30);
+	status = sa_pathsearch(sourceDomain, &nid, "/locations/lookupd");
 	syslock_unlock(rpcLock);
 	if (status == NI_OK)
 	{
@@ -674,7 +677,7 @@
 	if (didSetConfig) return NO;
 
 	didSetConfig = YES;
-	
+
 	if (initsource == configSourceAutomatic) initsource = src;
 
 	if (sourcePath != NULL) freeString(sourcePath);
@@ -697,6 +700,7 @@
 		{
 			syslock_lock(rpcLock);
 			sourceDomain = ni_shared_open(NULL, domain);
+			sourceDomainIsRawLocal = NO;
 			syslock_unlock(rpcLock);
 		}
 	}
@@ -802,6 +806,7 @@
 	sourcePath = NULL;
 	sourceDomainName = NULL;
 	sourceDomain = NULL;
+	sourceDomainIsRawLocal = NO;
 
 	return self;
 }
