@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -34,7 +33,7 @@
  */
 
 #import "MemoryWatchdog.h"
-#import "LUAgent.h"
+#import "CacheAgent.h"
 #import <string.h>
 
 @implementation MemoryWatchdog
@@ -61,6 +60,7 @@
 
 - (void)dealloc
 {
+	[Root setWatchdog:nil];
 	[list release];
 	syslock_free(listLock);
 	[super dealloc];
@@ -68,18 +68,23 @@
 
 - (void)showMemory:(FILE *)f
 {
-	int i, len;
+	int i, len, size, totalsize;
 	id obj;
 	char cachecode;
 	BOOL valid;
 
 	syslock_lock(listLock);
+
+	totalsize = 0;
 	len = [list count];
 
 	fprintf(f, "%d object%s in memory\n\n", len, (len == 1) ? "" : "s");
 	for (i = 0; i < len; i++)
 	{
 		obj = [list objectAtIndex:i];
+		size = [obj memorySize];
+		totalsize += size;
+
 		cachecode = ' ';
 		if ([self objectInCache:obj])
 		{
@@ -96,13 +101,15 @@
 			else cachecode = '.';
 		}
 
-		fprintf(f, "%5d %2d %c 0x%08x %s",
+		fprintf(f, "%5d %2d %c 0x%08x %8d %s",
 			i, [obj retainCount] - 1,
 			cachecode,
-			(unsigned int)obj, [obj banner]);
+			(unsigned int)obj, size, [obj banner]);
 		fprintf(f, "\n");
 
 	}
+
+	fprintf(f, "Total Size = %d\n", totalsize);
 	syslock_unlock(listLock);
 }
 
@@ -133,4 +140,26 @@
 	syslock_unlock(listLock);
 }
 
+- (unsigned int)totalMemory
+{
+	int i, len, size, totalsize;
+	id obj;
+
+	syslock_lock(listLock);
+
+	totalsize = 0;
+	len = [list count];
+
+	for (i = 0; i < len; i++)
+	{
+		obj = [list objectAtIndex:i];
+		size = [obj memorySize];
+		totalsize += size;
+	}
+	
+	syslock_unlock(listLock);
+
+	return totalsize;
+}
+
 @end
