@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -45,6 +44,7 @@
 #define ThreadStateIdle 1
 #define ThreadStateActive 2
 #define ThreadStateSleeping 3
+#define ThreadStateExitRequested 4
 
 @interface Thread : Root
 {
@@ -56,6 +56,7 @@
 	void *server;
 	unsigned long dataLen;
 	unsigned long state;
+	int notify_token;
 }
 
 + (void)shutdown;
@@ -73,16 +74,16 @@
 - (thread_type)thread;
 - (void)setThread:(thread_type)t;
 
-- (unsigned long)state;
-- (void)setState:(unsigned long)s;
+- (unsigned int)state;
+- (void)setState:(unsigned int)s;
 
 - (void)setName:(char *)n;
 
 - (void *)data;
 - (void)setData:(void *)d;
 
-- (unsigned long)dataLen;
-- (void)setDataLen:(unsigned long)l;
+- (unsigned int)dataLen;
+- (void)setDataLen:(unsigned int)l;
 
 - (void *)server;
 - (void)setServer:(void *)s;
@@ -119,7 +120,7 @@
 
 - (void)yield;
 
-- (void)sleep:(unsigned long)sec;
-- (void)usleep:(unsigned long)msec;
+- (void)sleep:(unsigned int)sec;
+- (void)usleep:(unsigned int)msec;
 
 @end
