@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -36,8 +35,12 @@
 #import "sys.h"
 #import <string.h>
 #import <unistd.h>
+#import <notify.h>
 #import <NetInfo/dsutil.h>
 
+extern uint32_t notify_set_state(int token, int state);
+extern uint32_t notify_register_plain(const char *name, int *out_token);
+
 static Thread **thread_list = NULL;
 static unsigned int initialized = 0;
 static unsigned int thread_count = 0;
@@ -332,6 +335,7 @@ launch_thread(launch_args *args)
 	dataLen = 0;
 	server = NULL;
 	state = ThreadStateInitial;
+	notify_token = -1;
 	unlock_threads();
 
 	return self;
@@ -340,17 +344,34 @@ launch_thread(launch_args *args)
 - (void)dealloc
 {
 	if (name != NULL) free(name);
+	if (notify_token != -1) notify_cancel(notify_token);
 	[super dealloc];
 }
 
-- (unsigned long)state
+- (unsigned int)state
 {
 	return state;
 }
 
-- (void)setState:(unsigned long)s
+- (void)setState:(unsigned int)s
 {
+	char *str;
+	int status;
+
 	state = s;
+
+	if (notify_token == -1)
+	{
+		str = NULL;
+		asprintf(&str, "self.thread.%lu", (unsigned long)thread);
+		if (str == NULL) return;
+	
+		status = notify_register_plain(str, &notify_token);
+		free(str);
+		if (status != NOTIFY_STATUS_OK) return;
+	}
+
+	notify_set_state(notify_token, s);
 }
 
 - (const char *)name
@@ -389,12 +410,12 @@ launch_thread(launch_args *args)
 	server = s;
 }
 
-- (unsigned long)dataLen
+- (unsigned int)dataLen
 {
 	return dataLen;
 }
 
-- (void)setDataLen:(unsigned long)l
+- (void)setDataLen:(unsigned int)l
 {
 	dataLen = l;
 }
@@ -480,6 +501,7 @@ launch_thread(launch_args *args)
 - (void)shouldTerminate:(BOOL)yn
 {
 	shouldTerminate = yn;
+	if (shouldTerminate) [self setState:ThreadStateExitRequested];
 }
 
 - (BOOL)shouldTerminate
@@ -503,7 +525,7 @@ launch_thread(launch_args *args)
 #endif
 }
 
-- (void)usleep:(unsigned long)msec
+- (void)usleep:(unsigned int)msec
 {
 	unsigned long oldState;
 	sys_msg_timeout_type snooze;
@@ -525,7 +547,7 @@ launch_thread(launch_args *args)
 	state = oldState;
 }
 
-- (void)sleep:(unsigned long)sec
+- (void)sleep:(unsigned int)sec
 {
 	if (sec == 0) return;
 	[self usleep:sec * 1000];
