@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -64,6 +63,7 @@
 #import <sys/resource.h>
 #import <signal.h>
 #import <notify.h>
+#import <dnsinfo.h>
 #import <mach/mig_errors.h>
 #import "_lu_types.h"
 
@@ -98,12 +98,14 @@ syslock *rpcLock = NULL;
 syslock *statsLock = NULL;
 char *portName = NULL;
 sys_port_type server_port = SYS_PORT_NULL;
+BOOL trace_enabled = NO;
+BOOL agent_debug_enabled = NO;
 BOOL debug_enabled = NO;
 BOOL statistics_enabled = NO;
 BOOL coredump_enabled = NO;
-BOOL lookup_local_interfaces = YES;
+BOOL aaaa_cutoff_enabled = YES;
 
-uint32_t gai_pref = GAI_S46;
+uint32_t gai_pref = GAI_P;
 uint32_t gai_wait = 2;
 
 /* Controller.m uses this global */
@@ -183,7 +185,7 @@ lookupd_startup()
 	char *name;
 	LUArray *config;
 	LUDictionary *cglobal;
-	int rf, nctoken;
+	int rf, nctoken, dnstoken;
 	uint32_t x;
 
 	rf = open("/dev/random", O_RDONLY, 0);
@@ -251,7 +253,10 @@ lookupd_startup()
 
 	nctoken = -1;
 	notify_register_signal(NETWORK_CHANGE_NOTIFICATION, SIGHUP, &nctoken);
-
+	
+	dnstoken = -1;
+	notify_register_signal(dns_configuration_notify_key(), SIGHUP, &dnstoken);
+	
 	if (!status)
 	{
 		if (debugMode)
