@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -74,9 +73,11 @@ extern uint32_t notify_register_plain(const char *name, int *out_token);
 
 #define NETINFO_NOTIFY_PREFIX "com.apple.system.netinfo"
 #define NETINFO_NOTIFY_SUFFIX "binding_change"
-static char *notification_name = NULL;
 
 static ni_status binding_status = NI_FAILED;
+static int notify_set_binding_status = 0;
+static int notify_token = -1;
+static char *notification_name = NULL;
 
 /*
  * Lookup "name"s address - returns it in net format
@@ -657,9 +658,37 @@ get_port(void *ni, char *proto)
 	return p;
 }
 
+#define BINDING_STATE_UNBOUND 0
+#define BINDING_STATE_BOUND 1
+#define BINDING_STATE_NETROOT 2
+
 ni_status
 get_binding_status(void)
 {
+	int state;
+
+	if (notify_set_binding_status != 0)
+	{
+		if ((notification_name == NULL) && (db_tag != NULL)) 
+		{
+			asprintf(&notification_name, "%s.%s.%s", NETINFO_NOTIFY_PREFIX, db_tag, NETINFO_NOTIFY_SUFFIX);
+		}
+		
+		if (notify_token == -1)
+		{
+			notify_register_plain(notification_name, &notify_token);
+		}
+
+		state = BINDING_STATE_UNBOUND;
+		if (binding_status == NI_OK) state = BINDING_STATE_BOUND;
+		else if (binding_status == NI_NETROOT) state = BINDING_STATE_NETROOT;
+	
+		notify_set_state(notify_token, state);
+		notify_post(notification_name);
+
+		notify_set_binding_status = 0;
+	}
+		
 	return binding_status;
 }
 
@@ -676,20 +705,24 @@ get_binding_status(void)
  * 2 - netroot
  */
  
-#define BINDING_STATE_UNBOUND 0
-#define BINDING_STATE_BOUND 1
-#define BINDING_STATE_NETROOT 2
-
 void
-set_binding_status(ni_status stat)
+set_binding_status(ni_status stat, int from_sighup)
 {
-	static int token = -1;
 	int old_state, new_state;
 
-	if ((notification_name == NULL) && (db_tag != NULL))
+	notify_set_binding_status = 0;
+
+	if (from_sighup == 0)
 	{
-		asprintf(&notification_name, "%s.%s.%s", NETINFO_NOTIFY_PREFIX, db_tag, NETINFO_NOTIFY_SUFFIX);
-		notify_register_plain(notification_name, &token);
+		if ((notification_name == NULL) && (db_tag != NULL)) 
+		{
+			asprintf(&notification_name, "%s.%s.%s", NETINFO_NOTIFY_PREFIX, db_tag, NETINFO_NOTIFY_SUFFIX);
+		}
+	
+		if (notify_token == -1)
+		{
+			notify_register_plain(notification_name, &notify_token);
+		}
 	}
 
 	old_state = BINDING_STATE_UNBOUND;
@@ -700,10 +733,17 @@ set_binding_status(ni_status stat)
 	if (stat == NI_OK) new_state = BINDING_STATE_BOUND;
 	else if (stat == NI_NETROOT) new_state = BINDING_STATE_NETROOT;
 
-	if ((old_state != new_state) && (notification_name != NULL))
+	if ((old_state != new_state) && (notify_token != -1))
 	{
-		notify_set_state(token, new_state);
-		notify_post(notification_name);
+		if (from_sighup == 0)
+		{
+			notify_set_state(notify_token, new_state);
+			notify_post(notification_name);
+		}
+		else
+		{
+			notify_set_binding_status = 1;
+		}
 	}
 
 	binding_status = stat;
