@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 2003 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -97,8 +96,8 @@ service_stat(const char *path, uid_t uid, gid_t gid)
 
 	status = stat(path, &sb);
 
-	seteuid(0);
-	setegid(g);
+	setegid(gid);
+	seteuid(uid);
 
 	if (status == 0) return 0;
 	if (errno == EACCES) return NOTIFY_STATUS_NOT_AUTHORIZED;
@@ -109,7 +108,7 @@ static int
 service_check_access(const char *path, int ftype, uid_t uid, gid_t gid)
 {
 	struct stat sb;
-	char *p, *str;
+	char *p, *str, rpath[PATH_MAX + 1];
 	int status;
 
 	if (uid == 0) return 0;
@@ -118,7 +117,9 @@ service_check_access(const char *path, int ftype, uid_t uid, gid_t gid)
 	/* Paths must be absolute */
 	if (path[0] != '/') return NOTIFY_STATUS_INVALID_REQUEST;
 
-	if (!strncasecmp(path, ZONEINFO_DIR, sizeof(ZONEINFO_DIR) - 1)) return 0;
+	memset(rpath, 0, sizeof(rpath));
+	if (realpath(path, rpath) == NULL) return 0;
+	if (!strncasecmp(rpath, ZONEINFO_DIR, sizeof(ZONEINFO_DIR) - 1)) return 0;
 
 	/* Root dir is readable */
 	if (path[1] == '\0') return 0;
@@ -167,7 +168,7 @@ service_check_access(const char *path, int ftype, uid_t uid, gid_t gid)
 	if (ftype == FS_TYPE_LINK)
 	{
 		/* Allow monitoring a symlink if the user has access to the link's directory */
-		
+
 		str = strdup(path);
 		p = strrchr(str, '/');
 		if (p == NULL)
@@ -234,7 +235,7 @@ service_open_file(int client_id, const char *name, const char *path, int flags,
 		if (f == NULL) return NOTIFY_STATUS_FAILED;
 		if (f->path == NULL) return NOTIFY_STATUS_FAILED;
 		if ((path != NULL) && (strcmp(path, f->path))) return NOTIFY_STATUS_INVALID_REQUEST;
-	}		
+	}
 
 	if (client_id == 0) return NOTIFY_STATUS_OK;
 
